from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from baricadr.client import Client

from future import standard_library

standard_library.install_aliases()


class FileClient(Client):
    """
    Manipulate files managed by Baricadr
    """

    def list(self, path, full=False, missing=False, max_depth=1, from_root=False):
        """
        List files available from a remote repository for a local path

        :type path: str
        :param path: Local path

        :type missing: bool
        :param missing: Only list files missing from the local path

        :type full: bool
        :param full: List full information for each file (size (in bytes), mtime, etc..)

        :type max_depth: int
        :param max_depth: Restrict to a max depth. Set to 0 for all files.

        :type from_root: bool
        :param from_root: Return full paths from root of the repo (instead of relative to given path)

        :rtype: list
        :return: List of file relative paths
        """

        body = {"path": path, "missing": missing, "max_depth": max_depth, "from_root": from_root, "full": full}
        return self._api_call("post", "list", body)

    def pull(self, path, email="", dry_run=False):
        """
        Launch a pull task

        :type path: str
        :param path: Local path to a missing file or folder

        :type email: str
        :param email: User email adress for notification

        :type dry_run: bool
        :param dry_run: Do not make any pull, just list changes that would be made

        :rtype: str
        :return: Id associated to the pull task
        """
        body = {"path": path, "dry_run": dry_run}
        if email:
            body['email'] = email

        return self._api_call("post", "pull", body)['task']

    def freeze(self, path, force=False, dry_run=False, email=""):
        """
        Launch a freeze task

        :type path: str
        :param path: Local path to a file or folder to freeze

        :type force: bool
        :param force: Force freezing, even if the freezing delay was not reached

        :type dry_run: bool
        :param dry_run: Do not make any deletion, just list changes that would be made

        :type email: str
        :param email: User email adress for notification

        :rtype: str
        :return: Id associated to the freeze task
        """
        body = {"path": path, "force": force, "dry_run": dry_run}

        if email:
            body['email'] = email

        return self._api_call("post", "freeze", body)['task']



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import re

from baricadr.exceptions import BaricadrApiError, BaricadrConnectionError, BaricadrNotImplementedError

from future import standard_library

import requests

standard_library.install_aliases()


class Client(object):
    """
    Base client class implementing methods to make queries to the server
    """

    def __init__(self, host, port, login, password, endpoints):
        self.host = host
        self.port = str(port)
        self.login = login
        self.password = password
        self.endpoints = endpoints

    def _api_call(self, call_type, endpoint_name, body={}):

        url = self._format_url(call_type, endpoint_name, body)

        auth = None
        if self.login and self.password:
            auth = (self.login, self.password)

        try:
            if call_type == "get":
                r = requests.get(url, json=body, auth=auth)
            elif call_type == "post":
                r = requests.post(url, json=body, auth=auth)

            if r.status_code == 400:
                raise BaricadrApiError("API call returned the following error: '{}'".format(r.json()['error']))
            elif r.status_code == 502:
                raise BaricadrApiError("Unknown server error")
            else:
                return r.json()

        except requests.exceptions.RequestException:
            raise BaricadrConnectionError("Cannot connect to {}:{}. Please check the connection.".format(self.host, self.port))

    def _format_url(self, call_type, endpoint_name, body):

        endpoint = self.endpoints.get(endpoint_name)
        if not endpoint:
            raise BaricadrNotImplementedError()

        # Fill parameters in the url
        if call_type == "get":
            groups = re.findall(r'<(.*?)>', endpoint)
            for group in groups:
                if group not in body:
                    raise BaricadrApiError("Missing get parameter " + group)
                endpoint = endpoint.replace("<{}>".format(group), body.get(group))

        return "http://{}:{}{}".format(self.host, self.port, endpoint)



from __future__ import absolute_import
import json
import os
import sys
import click

from .io import error
from .config import read_global_config, global_config_path, set_global_config_path, get_instance  # noqa, ditto
from barique import __version__  # noqa, ditto

CONTEXT_SETTINGS = dict(auto_envvar_prefix='BARIQUE', help_option_names=['-h', '--help'])


class Context(object):

    def __init__(self):
        self.verbose = False
        self.home = os.getcwd()
        self._global_config = None

    @property
    def global_config(self):
        if self._global_config is None:
            self._global_config = read_global_config()
        return self._global_config

    def log(self, msg, *args):
        """Logs a message to stderr."""
        if args:
            msg %= args
        click.echo(msg, file=sys.stderr)

    def vlog(self, msg, *args):
        """Logs a message to stderr only if verbose is enabled."""
        if self.verbose:
            self.log(msg, *args)

    def exit(self, exit_code):
        self.vlog("Exiting barique with exit code [%d]" % exit_code)
        sys.exit(exit_code)


pass_context = click.make_pass_decorator(Context, ensure=True)
cmd_folder = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                          'commands'))


def list_cmds():
    rv = []
    for filename in os.listdir(cmd_folder):
        if filename.endswith('.py') and \
           filename.startswith('cmd_'):
            rv.append(filename[len("cmd_"):-len(".py")])
    rv.sort()
    return rv


def list_subcmds(parent):
    rv = []
    for filename in os.listdir(os.path.join(cmd_folder, parent)):
        if filename.endswith('.py') and \
           not filename.startswith('__'):
            rv.append(filename[:-len(".py")])
    rv.sort()
    return rv


def name_to_command(parent, name):
    try:
        if sys.version_info[0] == 2:
            if parent:
                parent = parent.encode('ascii', 'replace')
            name = name.encode('ascii', 'replace')

        if parent:
            mod_name = 'barique.commands.%s.%s' % (parent, name)
        else:
            mod_name = 'barique.commands.cmd_' + name
        mod = __import__(mod_name, None, None, ['cli'])
    except ImportError as e:
        error("Problem loading command %s, exception %s" % (name, e))
        return
    return mod.cli


class bariqueCLI(click.MultiCommand):

    def list_commands(self, ctx):
        # We pre-calculate this so it works more nicely within packaged
        # versions of barique. Please feel free to fix this?

        commands = ['init', 'file', 'task']
        return commands

    def get_command(self, ctx, name):
        return name_to_command(None, name)


@click.command(cls=bariqueCLI, context_settings=CONTEXT_SETTINGS)
@click.version_option(__version__)
@click.option('-v', '--verbose', is_flag=True,
              help='Enables verbose mode.')
@click.option(
    "-i",
    "--instance",
    help='Name of instance in %s. This parameter can also be set via the environment variable BARICADR_INSTANCE' % global_config_path(),
    default='__default',
    show_default=True,
    required=True
)
@click.option(
    "--path", "-f",
    help="config file path",
    type=str
)
@pass_context
def barique(ctx, instance, verbose, path=None):
    """Command line wrappers around Baricadr functions."""
    # set config_path if provided
    if path is not None and len(path) > 0:
        set_global_config_path(path)
    # We abuse this, knowing that calls to one will fail.
    click.get_current_context()
    try:
        ctx.gi = get_instance(instance)
    except TypeError:
        pass

    ctx.verbose = verbose


def json_loads(data):
    """Load json data, allowing - to represent stdin."""
    if data is None:
        return ""

    if data == "-":
        return json.load(sys.stdin)
    elif os.path.exists(data):
        with open(data, 'r') as handle:
            return json.load(handle)
    else:
        return json.loads(data)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()


class BaricadrApiError(Exception):
    """Raised when the API returns an error"""
    pass


class BaricadrConnectionError(Exception):
    """Raised when the connection to the Barricadr server fails"""
    pass


class BaricadrNotImplementedError(Exception):
    """Raised when there are not endpoint associated with this function"""
    pass
